%
% This file performs the V-s iteration for estimating region-of-attraction
% for the GTM models.
%        1) Estimate ROA for the short-period with cubic pitch damping
%        2) Estimate ROA for the 4-state longitudinal poly model.
%

%==========================================================================
% Generate the GTM Models
Gen_GTMAnalModel;

%==========================================================================
% Choose which model to be analyzed and Degree of Lyapunov function

f2_N1 = 1;   % Open Loop Short period model with Shape Matrix N1
f2_N2 = 0;   % Open Loop Short period model with Shape Matrix N2
f4    = 0;   % Closed Loop Polynomial Longitudinal model

% Degree of Storage function
Vdeg = 2;

%==========================================================================
% Set up Model & ROA options according to chosen Model

if f2_N1
    
    % Model Information
    f = f2Cubic;
    xs = x(2:3);
    xt = xtrimC;
    ut = utrimC;
    
    % Define Shape Matrix N1
    Dmax = diag([20*d2r 50*d2r]);
    N1 = inv(Dmax^2);
    p = xs'*N1*xs;
    
    % ROA Multipliers & options
    z1maxd = ceil((Vdeg-p.maxdeg)/2);
    z1 = monomials(xs, 0:z1maxd );
    z2 = monomials(xs, 1:2 );
        
elseif f2_N2
    
    % Model Information
    f = f2Cubic;
    xs = x(2:3);
    xt = xtrimC;
    ut = utrimC;
    
    % Define Shape Matrix N2
    Dmax = diag([10*d2r 50*d2r]);
    N2 = inv(Dmax^2);
    p = xs'*N2*xs;
    
    % ROA Multipliers & options
    z1maxd = ceil((Vdeg-p.maxdeg)/2);
    z1 = monomials(xs, 0:z1maxd );
    z2 = monomials(xs, 1:2 );
    
else
    
    % Change V (velocity) variable to U, (ROA estimation V is Lyap function)
    pvar U;
    
    % Model Information: State Normalization & Scaling
    Dmax = diag([20 20*d2r 50*d2r 20*d2r]);
    f = inv(Dmax)*subs(f4Clp,x,Dmax*x);
    xs = subs(x,V,U); % Change pvar V to U to avoid confusion with Lyap function
    f = subs(f,x,xs);
    xt = xtrim4;
    ut = utrim4;
    
    % Define Shape Matrix
    p = xs'*xs;
    
    % ROA Multipliers & options
    z1maxd = ceil((Vdeg-p.maxdeg)/2);
    z1 = monomials(xs, 0:z1maxd );
    z2 = monomials(xs, 1:3 );
    
end

% Create Lyapunov Function
switch Vdeg
    case 2
        zV = monomials(xs,Vdeg);
    case 4
        zV = monomials(xs,2:Vdeg);
    case 6
        zV = monomials(xs,2:Vdeg);
end

% Create ROAOPTIONS

NstepBis = 25;
NstepMinTrace = 0;
L2 = 1e-6*(xs'*xs);
ropt = roaoptions(f,xs,'zV',zV,'p',p,'L2',L2,'NstepMinTrace',NstepMinTrace,...
    'z1',z1,'z2',z2,'NstepBis',NstepBis,'display','on');

%==========================================================================
% Run ROAEST to estimate ROA
[beta,V,gamma,s1,s2,iter] = roaest(f,xs,ropt);


%==========================================================================
% Plot Contours  of alpha / q

if f2_N1 || f2_N2
    pC = ropt.p;
else
    Dmax_alp_q = diag([Dmax(2,2) Dmax(3,3)]);
    scal = diag([0 inv(Dmax(2,2))  inv(Dmax(3,3)) 0]);
    pC = subs(ropt.p, xs, scal*xs);
end

C1 = pcontour(pC/r2d,beta*r2d);

figure1 = figure;
axes('Parent',figure1,'FontSize',18);
plot(C1(1,2:end)+ xt(2)*r2d, C1(2,2:end)+xt(3)*r2d,'-b','LineWidth',1.5)
hold on
plot(xt(2)*r2d,xt(3)*r2d,'x','MarkerSize',16,'LineWidth',3)
xlabel('\alpha (deg)','FontSize',14)
ylabel('q (deg/s)','FontSize',14)
grid on
hold off

return